@@ -9,8 +9,8 @@
 #include <memory>
 
 #include "base/macros.h"
-#include "base/memory/ptr_util.h"
 #include "base/strings/utf_string_conversions.h"
+#include "build/build_config.h"
 #include "ui/base/ui_base_types.h"
 #include "ui/events/test/event_generator.h"
 #include "ui/views/controls/menu/menu_controller.h"
@@ -64,8 +64,12 @@ class MenuRunnerTest : public ViewsTestBase {
   MenuRunnerTest();
   ~MenuRunnerTest() override;
 
-  // Initializes a MenuRunner with |run_types|. It takes ownership of
-  // |menu_item_view_|.
+  // Initializes the delegates and views needed for a menu. It does not create
+  // the MenuRunner.
+  void InitMenuViews();
+
+  // Initializes all delegates and views needed for a menu. A MenuRunner is also
+  // created with |run_types|, it takes ownership of |menu_item_view_|.
   void InitMenuRunner(int32_t run_types);
 
   MenuItemView* menu_item_view() { return menu_item_view_; }
@@ -74,12 +78,11 @@ class MenuRunnerTest : public ViewsTestBase {
   Widget* owner() { return owner_.get(); }
 
   // ViewsTestBase:
-  void SetUp() override;
   void TearDown() override;
 
  private:
-  // Owned by MenuRunner.
-  MenuItemView* menu_item_view_;
+  // Owned by menu_runner_.
+  MenuItemView* menu_item_view_ = nullptr;
 
   std::unique_ptr<TestMenuDelegate> menu_delegate_;
   std::unique_ptr<MenuRunner> menu_runner_;
@@ -92,12 +95,7 @@ MenuRunnerTest::MenuRunnerTest() {}
 
 MenuRunnerTest::~MenuRunnerTest() {}
 
-void MenuRunnerTest::InitMenuRunner(int32_t run_types) {
-  menu_runner_.reset(new MenuRunner(menu_item_view_, run_types));
-}
-
-void MenuRunnerTest::SetUp() {
-  ViewsTestBase::SetUp();
+void MenuRunnerTest::InitMenuViews() {
   menu_delegate_.reset(new TestMenuDelegate);
   menu_item_view_ = new MenuItemView(menu_delegate_.get());
   menu_item_view_->AppendMenuItemWithLabel(1, base::ASCIIToUTF16("One"));
@@ -111,8 +109,14 @@ void MenuRunnerTest::SetUp() {
   owner_->Show();
 }
 
+void MenuRunnerTest::InitMenuRunner(int32_t run_types) {
+  InitMenuViews();
+  menu_runner_.reset(new MenuRunner(menu_item_view_, run_types));
+}
+
 void MenuRunnerTest::TearDown() {
-  owner_->CloseNow();
+  if (owner_)
+    owner_->CloseNow();
   ViewsTestBase::TearDown();
 }
 
@@ -163,6 +167,7 @@ TEST_F(MenuRunnerTest, LatinMnemonic) {
   if (IsMus())
     return;
 
+  views::test::DisableMenuClosureAnimations();
   InitMenuRunner(0);
   MenuRunner* runner = menu_runner();
   runner->RunMenuAt(owner(), nullptr, gfx::Rect(), MENU_ANCHOR_TOPLEFT,
@@ -171,6 +176,7 @@ TEST_F(MenuRunnerTest, LatinMnemonic) {
 
   ui::test::EventGenerator generator(GetContext(), owner()->GetNativeWindow());
   generator.PressKey(ui::VKEY_O, 0);
+  views::test::WaitForMenuClosureAnimation();
   EXPECT_FALSE(runner->IsRunning());
   TestMenuDelegate* delegate = menu_delegate();
   EXPECT_EQ(1, delegate->execute_command_id());
@@ -178,14 +184,16 @@ TEST_F(MenuRunnerTest, LatinMnemonic) {
   EXPECT_NE(nullptr, delegate->on_menu_closed_menu());
 }
 
+#if !defined(OS_WIN)
 // Tests that a key press on a non-US keyboard layout activates the correct menu
-// item.
+// item. Disabled on Windows because a WM_CHAR event does not activate an item.
 TEST_F(MenuRunnerTest, NonLatinMnemonic) {
   // TODO: test uses GetContext(), which is not applicable to aura-mus.
   // http://crbug.com/663809.
   if (IsMus())
     return;
 
+  views::test::DisableMenuClosureAnimations();
   InitMenuRunner(0);
   MenuRunner* runner = menu_runner();
   runner->RunMenuAt(owner(), nullptr, gfx::Rect(), MENU_ANCHOR_TOPLEFT,
@@ -195,12 +203,14 @@ TEST_F(MenuRunnerTest, NonLatinMnemonic) {
   ui::test::EventGenerator generator(GetContext(), owner()->GetNativeWindow());
   ui::KeyEvent key_press(0x062f, ui::VKEY_N, 0);
   generator.Dispatch(&key_press);
+  views::test::WaitForMenuClosureAnimation();
   EXPECT_FALSE(runner->IsRunning());
   TestMenuDelegate* delegate = menu_delegate();
   EXPECT_EQ(2, delegate->execute_command_id());
   EXPECT_EQ(1, delegate->on_menu_closed_called());
   EXPECT_NE(nullptr, delegate->on_menu_closed_menu());
 }
+#endif  // !defined(OS_WIN)
 
 // Tests that attempting to nest a menu within a drag-and-drop menu does not
 // cause a crash. Instead the drag and drop action should be canceled, and the
@@ -361,7 +371,21 @@ TEST_F(MenuRunnerWidgetTest, ClearsMouseHandlerOnRun) {
   EXPECT_EQ(1, second_event_count_view->GetEventCount(ui::ET_MOUSE_PRESSED));
 }
 
-typedef MenuRunnerTest MenuRunnerImplTest;
+class MenuRunnerImplTest : public MenuRunnerTest {
+ public:
+  MenuRunnerImplTest() {}
+  ~MenuRunnerImplTest() override {}
+
+  void SetUp() override;
+
+ private:
+  DISALLOW_COPY_AND_ASSIGN(MenuRunnerImplTest);
+};
+
+void MenuRunnerImplTest::SetUp() {
+  MenuRunnerTest::SetUp();
+  InitMenuViews();
+}
 
 // Tests that when nested menu runners are destroyed out of order, that
 // MenuController is not accessed after it has been destroyed. This should not
@@ -466,6 +490,7 @@ void MenuRunnerDestructionTest::SetUp() {
   views_delegate_ = views_delegate.get();
   set_views_delegate(std::move(views_delegate));
   MenuRunnerTest::SetUp();
+  InitMenuViews();
 }
 
 // Tests that when ViewsDelegate is released that a nested Cancel of the
