@@ -59,10 +59,6 @@ class OSExchangeData;
 class ThemeProvider;
 }  // namespace ui
 
-namespace wm {
-enum class ShadowElevation;
-}
-
 namespace views {
 
 class DesktopWindowTreeHost;
@@ -243,7 +239,9 @@ class VIEWS_EXPORT Widget : public internal::NativeWidgetDelegate,
     ShadowType shadow_type;
     // A hint about the size of the shadow if the type is SHADOW_TYPE_DROP. May
     // be ignored on some platforms. No value indicates no preference.
-    base::Optional<wm::ShadowElevation> shadow_elevation;
+    base::Optional<int> shadow_elevation;
+    // The window corner radius. May be ignored on some platforms.
+    base::Optional<int> corner_radius;
     // Specifies that the system default caption and icon should not be
     // rendered, and that the client area should be equivalent to the window
     // area. Only used on some platforms (Windows and Linux).
@@ -278,8 +276,9 @@ class VIEWS_EXPORT Widget : public internal::NativeWidgetDelegate,
     // hierarchy this widget should be placed. (This is separate from |parent|;
     // if you pass a RootWindow to |parent|, your window will be parented to
     // |parent|. If you pass a RootWindow to |context|, we ask that RootWindow
-    // where it wants your window placed.) NULL is not allowed if you are using
-    // aura.
+    // where it wants your window placed.) Nullptr is not allowed on Windows and
+    // Linux. Nullptr is allowed on Chrome OS, which will place the window on
+    // the default desktop for new windows.
     gfx::NativeWindow context;
     // If true, forces the window to be shown in the taskbar, even for window
     // types that do not appear in the taskbar by default (popup and bubble).
@@ -328,14 +327,6 @@ class VIEWS_EXPORT Widget : public internal::NativeWidgetDelegate,
   // during application shutdown when the last non-secondary widget is closed.
   static void CloseAllSecondaryWidgets();
 
-  // Converts a rectangle from one Widget's coordinate system to another's.
-  // Returns false if the conversion couldn't be made, because either these two
-  // Widgets do not have a common ancestor or they are not on the screen yet.
-  // The value of |*rect| won't be changed when false is returned.
-  static bool ConvertRect(const Widget* source,
-                          const Widget* target,
-                          gfx::Rect* rect);
-
   // Retrieves the Widget implementation associated with the given
   // NativeView or Window, or NULL if the supplied handle has no associated
   // Widget.
@@ -429,6 +420,9 @@ class VIEWS_EXPORT Widget : public internal::NativeWidgetDelegate,
   // fit the entire size of the RootView. The RootView takes ownership of this
   // View, unless it is set as not being parent-owned.
   void SetContentsView(View* view);
+
+  // NOTE: This may not be the same view as WidgetDelegate::GetContentsView().
+  // See RootView::GetContentsView().
   View* GetContentsView();
 
   // Returns the bounds of the Widget in screen coordinates.
@@ -450,9 +444,8 @@ class VIEWS_EXPORT Widget : public internal::NativeWidgetDelegate,
   // Sizes the window to the specified size and centerizes it.
   void CenterWindow(const gfx::Size& size);
 
-  // Like SetBounds(), but ensures the Widget is fully visible on screen,
-  // resizing and/or repositioning as necessary. This is only useful for
-  // non-child widgets.
+  // Like SetBounds(), but ensures the Widget is fully visible on screen or
+  // parent widget, resizing and/or repositioning as necessary.
   void SetBoundsConstrained(const gfx::Rect& bounds);
 
   // Sets whether animations that occur when visibility is changed are enabled.
@@ -554,6 +547,12 @@ class VIEWS_EXPORT Widget : public internal::NativeWidgetDelegate,
   // underlying windowing system.
   void SetOpacity(float opacity);
 
+  // Sets the aspect ratio of the widget's content, which will be maintained
+  // during interactive resizing. This size disregards title bar and borders.
+  // Once set, some platforms ensure the content will only size to integer
+  // multiples of |aspect_ratio|.
+  void SetAspectRatio(const gfx::SizeF& aspect_ratio);
+
   // Flashes the frame of the window to draw attention to it. Currently only
   // implemented on Windows for non-Aura.
   void FlashFrame(bool flash);
@@ -791,7 +790,7 @@ class VIEWS_EXPORT Widget : public internal::NativeWidgetDelegate,
   bool CanActivate() const override;
   bool IsAlwaysRenderAsActive() const override;
   void SetAlwaysRenderAsActive(bool always_render_as_active) override;
-  void OnNativeWidgetActivationChanged(bool active) override;
+  bool OnNativeWidgetActivationChanged(bool active) override;
   void OnNativeFocus() override;
   void OnNativeBlur() override;
   void OnNativeWidgetVisibilityChanging(bool visible) override;
@@ -860,6 +859,7 @@ class VIEWS_EXPORT Widget : public internal::NativeWidgetDelegate,
   friend class ButtonTest;
   friend class TextfieldTest;
   friend class ViewAuraTest;
+  friend void DisableActivationChangeHandlingForTests();
 
   // Persists the window's restored position and "show" state using the
   // window delegate.
@@ -885,6 +885,8 @@ class VIEWS_EXPORT Widget : public internal::NativeWidgetDelegate,
   // layer.
   const View::Views& GetViewsWithLayers();
 
+  static bool g_disable_activation_change_handling_;
+
   internal::NativeWidgetPrivate* native_widget_;
 
   base::ObserverList<WidgetObserver> observers_;
