@@ -63,6 +63,7 @@
 
 #include "chrome/browser/custom_handlers/protocol_handler_registry_factory.h"
 #include "components/web_cache/browser/web_cache_manager.h"
+#include "content/browser/frame_host/render_frame_host_impl.h"
 #include "content/browser/renderer_host/render_widget_host_impl.h"
 #include "content/public/browser/browser_context.h"
 #include "content/public/browser/invalidate_type.h"
@@ -102,6 +103,7 @@ WebContentsDelegateQt::WebContentsDelegateQt(content::WebContents *webContents,
     , m_lastReceivedFindReply(0)
     , m_faviconManager(new FaviconManager(webContents, adapterClient))
     , m_lastLoadProgress(-1)
+    , m_frameFocusedObserver(adapterClient)
 {
     webContents->SetDelegate(this);
     Observe(webContents);
@@ -223,6 +225,11 @@ void WebContentsDelegateQt::AddNewContents(content::WebContents* source, std::un
 {
     Q_UNUSED(source)
     QSharedPointer<WebContentsAdapter> newAdapter = createWindow(std::move(new_contents), disposition, initial_pos, user_gesture);
+    // Chromium can forget to pass user-agent override settings to new windows (see QTBUG-61774 and QTBUG-76249),
+    // so set it here. Note the actual value doesn't really matter here. Only the second value does, but we try
+    // to give the correct user-agent anyway.
+    if (newAdapter)
+        newAdapter->webContents()->SetUserAgentOverride(newAdapter->profileAdapter()->httpUserAgent().toStdString(), true);
     if (newAdapter && !newAdapter->isInitialized())
         newAdapter->loadDefault();
     if (was_blocked)
@@ -253,11 +260,30 @@ void WebContentsDelegateQt::HandleKeyboardEvent(content::WebContents *, const co
         m_viewClient->unhandledKeyEvent(reinterpret_cast<QKeyEvent *>(event.os_event));
 }
 
+void WebContentsDelegateQt::RenderFrameCreated(content::RenderFrameHost *render_frame_host)
+{
+    content::FrameTreeNode *node = static_cast<content::RenderFrameHostImpl *>(render_frame_host)->frame_tree_node();
+    m_frameFocusedObserver.addNode(node);
+}
+
 void WebContentsDelegateQt::RenderFrameDeleted(content::RenderFrameHost *render_frame_host)
 {
     m_loadingErrorFrameList.removeOne(render_frame_host->GetRoutingID());
 }
 
+void WebContentsDelegateQt::RenderFrameHostChanged(content::RenderFrameHost *old_host, content::RenderFrameHost *new_host)
+{
+    if (old_host) {
+        content::FrameTreeNode *old_node = static_cast<content::RenderFrameHostImpl *>(old_host)->frame_tree_node();
+        m_frameFocusedObserver.removeNode(old_node);
+    }
+
+    if (new_host) {
+        content::FrameTreeNode *new_node = static_cast<content::RenderFrameHostImpl *>(new_host)->frame_tree_node();
+        m_frameFocusedObserver.addNode(new_node);
+    }
+}
+
 void WebContentsDelegateQt::RenderViewHostChanged(content::RenderViewHost *, content::RenderViewHost *newHost)
 {
     if (newHost && newHost->GetWidget() && newHost->GetWidget()->GetView()) {
@@ -484,7 +510,7 @@ void WebContentsDelegateQt::FindReply(content::WebContents *source, int request_
     Q_UNUSED(source)
     Q_UNUSED(selection_rect)
     Q_UNUSED(active_match_ordinal)
-    if (final_update) {
+    if (final_update && request_id > m_lastReceivedFindReply) {
         m_lastReceivedFindReply = request_id;
         m_viewClient->didFindText(request_id, number_of_matches);
     }
@@ -684,6 +710,12 @@ void WebContentsDelegateQt::UnregisterProtocolHandler(content::WebContents *webC
     registry->RemoveHandler(handler);
 }
 
+bool WebContentsDelegateQt::TakeFocus(content::WebContents *source, bool reverse)
+{
+    Q_UNUSED(source);
+    return m_viewClient->passOnFocus(reverse);
+}
+
 FaviconManager *WebContentsDelegateQt::faviconManager()
 {
     return m_faviconManager.data();
@@ -698,4 +730,42 @@ WebContentsAdapter *WebContentsDelegateQt::webContentsAdapter() const
     return m_viewClient->webContentsAdapter();
 }
 
+
+FrameFocusedObserver::FrameFocusedObserver(WebContentsAdapterClient *adapterClient)
+    : m_viewClient(adapterClient)
+{}
+
+void FrameFocusedObserver::addNode(content::FrameTreeNode *node)
+{
+   if (m_observedNodes.contains(node))
+       return;
+
+   node->AddObserver(this);
+   m_observedNodes.append(node);
+}
+
+void FrameFocusedObserver::removeNode(content::FrameTreeNode *node)
+{
+    node->RemoveObserver(this);
+    m_observedNodes.removeOne(node);
+}
+
+void FrameFocusedObserver::OnFrameTreeNodeFocused(content::FrameTreeNode *node)
+{
+    Q_UNUSED(node);
+    m_viewClient->updateEditActions();
+}
+
+void FrameFocusedObserver::OnFrameTreeNodeDestroyed(content::FrameTreeNode *node)
+{
+    m_observedNodes.removeOne(node);
+    m_viewClient->updateEditActions();
+}
+
+FrameFocusedObserver::~FrameFocusedObserver()
+{
+    for (content::FrameTreeNode *node : m_observedNodes)
+        node->RemoveObserver(this);
+}
+
 } // namespace QtWebEngineCore
